#!/bin/sh

set -eu

fail_exit() {
	echo "FAILED"
}
trap fail_exit EXIT

alias grep='grep -q'

MANAGER="gdbus call --system --dest net.connman --object-path / --method net.connman.Manager"

test_manager() {
	echo test_manager
	${MANAGER}.GetProperties > /dev/null
}

test_offlinemode() {
	echo test_offlinemode
	${MANAGER}.SetProperty "OfflineMode" "<true>" >> /dev/null
	${MANAGER}.GetProperties | grep "'OfflineMode': <true>"
	${MANAGER}.SetProperty "OfflineMode" "<false>" >> /dev/null
	${MANAGER}.GetProperties | grep "'OfflineMode': <false>"
}

test_technology_enable() {
	local OBJECT_PATH=$1
	echo test_technology_enable ${OBJECT_PATH}

	TECHNOLOGY="gdbus call --system --dest net.connman --object-path ${OBJECT_PATH} --method net.connman.Technology"

	# Enable technology if not enabled
	${TECHNOLOGY}.GetProperties | grep "Powered': <true>" ||
		${TECHNOLOGY}.SetProperty "Powered" "<true>"

	# Disable technology
	${TECHNOLOGY}.SetProperty "Powered" "<false>" >> /dev/null

	# Check it is disabled
	${TECHNOLOGY}.GetProperties |  grep "Powered': <false>" ||
		exit 1

	# Enable again
	${TECHNOLOGY}.SetProperty "Powered" "<true>" >> /dev/null

	sleep 1

	# Check it was enabled properly
	${TECHNOLOGY}.GetProperties | grep "Powered': <true>"
}


test_technology_tethering() {
	local OBJECT_PATH=$1
	echo test_technology_tethering ${OBJECT_PATH}
	TECHNOLOGY="gdbus call --system --dest net.connman --object-path ${OBJECT_PATH} --method net.connman.Technology"
	if ${TECHNOLOGY}.GetProperties | grep "'Tethering': <true>" ; then
		${TECHNOLOGY}.GetProperties | grep "TetheringIdentifier" ||
			exit 1
		${TECHNOLOGY}.GetProperties | grep "TetheringPassphrase" ||
			exit 1
	fi
}

test_technology_check_property() {
	local PROP_NAME=$1
	case ${PROP_NAME} in
	Name|Type|Powered|Connected|Tethering) ;;
	*) "Invalid property ${PROP_NAME}" ; exit 1 ;;
	esac
}

test_technologies() {
	echo test_technologies
	TECHNOLOGIES=`${MANAGER}.GetTechnologies`
	for WORD in ${TECHNOLOGIES} ; do
		case ${WORD} in
		"'/net/connman/technology/"*)
			OBJECT_PATH=`echo ${WORD} | sed -e "s/.*'\(.*\)'.*/\1/"`
			test_technology_enable ${OBJECT_PATH}
			test_technology_tethering ${OBJECT_PATH}
			;;
		*":") test_technology_check_property `echo ${WORD} | sed "s/.*'\(.*\)'.*/\1/"`;;
		esac
	done
}

test_service_properties() {
	local OBJECT_PATH=$1
	echo test_service_properties ${OBJECT_PATH}

	SERVICE="gdbus call --system --dest net.connman --object-path ${OBJECT_PATH} --method net.connman.Service"

	# Disconnect service
	${SERVICE}.Disconnect  >> /dev/null || true
	sleep 3

	# Make sure service is connected
	${SERVICE}.Connect >> /dev/null

	${SERVICE}.GetProperties | grep "State"
	${SERVICE}.GetProperties | grep "Name"
	${SERVICE}.GetProperties | grep "Type"
	${SERVICE}.GetProperties | grep "Favorite"
	${SERVICE}.GetProperties | grep "Immutable"
	${SERVICE}.GetProperties | grep "AutoConnect"
	${SERVICE}.GetProperties | grep "Nameservers"
	${SERVICE}.GetProperties | grep "Nameservers.Configuration"
	${SERVICE}.GetProperties | grep "Timeservers"
	${SERVICE}.GetProperties | grep "Timeservers.Configuration"
	${SERVICE}.GetProperties | grep "Domains"
	${SERVICE}.GetProperties | grep "Domains.Configuration"
	${SERVICE}.GetProperties | grep "IPv4"
	${SERVICE}.GetProperties | grep "IPv4.Configuration"
	${SERVICE}.GetProperties | grep "IPv6"
	${SERVICE}.GetProperties | grep "IPv6.Configuration"
	${SERVICE}.GetProperties | grep "Proxy"
	${SERVICE}.GetProperties | grep "Proxy.Configuration"
	${SERVICE}.GetProperties | grep "Provider"
	${SERVICE}.GetProperties | grep "Ethernet"

	${SERVICE}.SetProperty "Timeservers.Configuration" "<['0.fedora.pool.ntp.org', '1.fedora.pool.ntp.org', '2.fedora.pool.ntp.org']>" >> /dev/null

	${SERVICE}.SetProperty "Nameservers.Configuration" "<['4.4.4.4', '8.8.8.8']>" >> /dev/null

	${SERVICE}.SetProperty "AutoConnect" "<false>" >> /dev/null
	${SERVICE}.GetProperties | grep "'AutoConnect': <false>"

	${SERVICE}.SetProperty "AutoConnect" "<true>" >> /dev/null
	${SERVICE}.GetProperties | grep "'AutoConnect': <true>"
}

test_ethernet() {
	echo test_ethernet

	SERVICES=`${MANAGER}.GetServices`
	for WORD in ${SERVICES} ; do
		case ${WORD} in
		"'/net/connman/service/"*)
			OBJECT_PATH=`echo ${WORD} | sed -e "s/.*'\(.*\)'.*/\1/"`
			test_service_properties ${OBJECT_PATH}
			;;
		esac
	done
}

test_clock() {
	echo test_clock

	CLOCK="gdbus call --system --dest net.connman --object-path / --method net.connman.Clock"

	${CLOCK}.GetProperties | grep "TimeUpdates"
	${CLOCK}.GetProperties | grep "Timezone"
	${CLOCK}.GetProperties | grep "Timeservers"
	${CLOCK}.GetProperties | grep "TimezoneUpdates"
	TIME=$(${CLOCK}.GetProperties | sed -e "s/.*'Time': <uint64 \([0-9]*\)>.*/\1/")

	${CLOCK}.SetProperty "TimeUpdates" "<'manual'>" >> /dev/null
	${CLOCK}.SetProperty "Time" "<uint64 ${TIME}>" >> /dev/null
	${CLOCK}.SetProperty "Timeservers" "<['0.fedora.pool.ntp.org', '1.fedora.pool.ntp.org', '2.fedora.pool.ntp.org']>" >> /dev/null
	${CLOCK}.SetProperty "TimeUpdates" "<'auto'>" >> /dev/null
	${CLOCK}.SetProperty "TimezoneUpdates" "<'manual'>" >> /dev/null
	${CLOCK}.SetProperty "Timezone" "<'America/Vancouver'>" >> /dev/null
	${CLOCK}.SetProperty "TimezoneUpdates" "<'auto'>" >> /dev/null
}

test_manager
test_offlinemode
test_technologies
test_ethernet
test_clock

trap '' EXIT
echo "PASSED"
